function [ params ] = holoLens1Params(params)

    params.dataset.query.dirname = 'query-HoloLens1';
    params.dataset.query.dir = fullfile(params.dataset.dir, params.dataset.query.dirname); % NOTE: it cannot be extracted to setupParams.m, because we need it in here already
    params.dataset.query.dslevel = 8^-1;

    env = environment();

    if strcmp(env, 'laptop')
        params.holoLens.dir = '/Volumes/GoogleDrive/Můj disk/ARTwin/personal/lucivpav/HoloLens sequences';
    elseif strcmp(env, 'cmp')
        params.holoLens.dir = '/mnt/datagrid/personal/lucivpav/HoloLens sequences';
    end

    params.holoLens.measurement.path = fullfile(params.holoLens.dir, 'measurement1.txt');
    params.holoLens.recording.dir = fullfile(params.holoLens.dir, 'HoloLensRecording__2020_04_23__09_53_01');
    params.holoLens.query.dir = fullfile(params.holoLens.recording.dir, 'pv');
    params.holoLens.poses.path = fullfile(params.holoLens.recording.dir, 'pv_locationData.csv');
    params.HoloLensPoses.dir = fullfile(params.dataset.query.dir, 'HoloLensPoses');
    params.HoloLensProjectedPointCloud.dir = fullfile(params.dataset.query.dir, 'HoloLensProjectedPointCloud');
    params.HoloLensTranslationDelay = 6; % in frames, w.r.t. reference poses
    params.HoloLensOrientationDelay = 4; % in frames, w.r.t. reference poses
    params.sequence.length = 5; % data is sequential and we want to use HoloLens pose estimates to improve localization

    % NOTE: some reference poses are wrong due to Vicon error, blacklist them
    params.blacklistedQueryInd = [103:109, 162, 179:188, 191:193, 286:288];

    interestingQueryInd = [1,2,3,4,5,6];
    % naive params from P3P:
    optimalRotations = {[-96.6116, 4.2738, -5.5309],
                        [-96.5095, 4.7007, -7.0947],
                        [-97.3403, 5.0600, -6.0764],
                        [-97.3304, 5.2070, -4.7746],
                        [-97.3062, 4.3380, -4.9227],
                        [-97.3005, 4.8393, -5.7490]};
    params.optimal.camera.rotation.wrt.marker = containers.Map(interestingQueryInd, optimalRotations);
    %params.camera.rotation.wrt.marker = [-97.0664 4.7364 -5.6914]; % generic from findOptimalParamsForInterestingQueries
        % Standard deviation of optimal rotations: 0.3935 0.3770 0.8459
    params.camera.rotation.wrt.marker = [-97.5664, 4.7364, -5.1914]; % from buildRawPoses

    interestingQueryInd = [1,2,3,4,5,6];
    % naive params from P3P:
    optimalTranslations = {[4.3968; 16.1647; -7.4257],
                            [-2.4515; 9.6106; -5.0424],
                            [4.6812; 10.4508; -9.7955],
                            [-1.6800; 13.7664; -9.4822],
                            [4.0147; 17.9644; -6.8934],
                            [-0.1598; 9.5486; -7.8150]};
    params.optimal.camera.origin.relative.wrt.marker = containers.Map(interestingQueryInd, optimalTranslations);
    %params.camera.origin.relative.wrt.marker = [1.4669; 12.9176; -7.7423]; % generic from findOptimalParamsForInterestingQueries
        % Standard deviation of optimal origins: 3.2653 3.6085 1.7526
    params.camera.origin.relative.wrt.marker = [1.4669; 12.9176; -4.7423]; % from buildRawPoses, NOTE: z is a bit strange
    params.camera.originConstant = 0.023;
    params.camera.origin.wrt.marker = params.camera.originConstant * params.camera.origin.relative.wrt.marker;
    params.camera.sensor.size = [756, 1344]; % height, width
    params.camera.fl = 1038; % in pixels; this is not worse than the previous fl
    %params.HoloLensViconSyncConstant = 10.7 * 1000; % [ms]
    params.HoloLensViconSyncConstant = 10.7 * 1000; % [ms]; from buildRawPoses
    
    %% interesting queries and corresponding matches %%
    params.interestingQueries = ["00132321090555753820.jpg", ... % aka query 1 (1.jpg)
                                 "00132321090868821963.jpg", ... % aka query 2 (94.jpg)
                                 "00132321091341754646.jpg", ... % aka query 3 (237.jpg)
                                 "00132321091488297196.jpg", ... % aka query 4 (281.jpg)
                                 "00132321091068652686.jpg", ... % aka query 5 (155.jpg)
                                 "00132321091211864676.jpg"]; % aka query 6 (198.jpg)
                                    % TODO: 00132321091305119025.jpg
    % interestingPointsQuery{i} are 2D projections of points in interestingPointsPC{i}.
    % i is the i-th query in params.interestingQueries
    params.interestingPointsPC{1} = [-3.5444, 2.6774, -3.2116; ...
                                        -11.2066, 2.3874, 1.0484; ...
                                        -2.7544, 3.5366, 1.5284; ...
                                        -5.5544, 3.5336, -1.2016; ...
                                        -7.0966, 0.6574, 1.0484; ...
                                        -2.2044, 3.5442, -2.7116; ...
                                        -11.2115, 3.4974, -3.1916; ...
                                        -7.3193, 0.0774, 2.6184; ...
                                        -0.9344, 3.5344, -0.8916]';
    params.interestingPointsQuery{1} = [1046, 432; ...
                                        398, 502; ...
                                        228, 151; ...
                                        671, 330; ...
                                        353, 680; ...
                                        1099, 218; ...
                                        741, 438; ...
                                        152, 743; ...
                                        844, 49]';

    params.interestingPointsPC{2} = [-6.6944, 0.7574, -2.3250; ...
                                        -8.2644, 0.6513, -2.4816; ...
                                        -5.0944, 2.4974, -3.3415; ...
                                        -5.2344, 1.0874, -3.3414; ...
                                        -7.1374, 0.1474, -2.4916; ...
                                        -6.7544, 0.0036, -3.2716; ...
                                        -9.3044, 1.2918, -3.3616; ... % not so trustworthy
                                        -7.3284, 0.0574, -3.2935; ...
                                        -5.4044, 0.8974, -2.5116]'; % not trustworthy
    params.interestingPointsQuery{2} = [271, 286; ...
                                        166, 138; ...
                                        1105, 361; ...
                                        739, 652; ...
                                        164, 383; ...
                                        319, 530; ...
                                        346, 43; ...
                                        288, 436; ...
                                        531, 554]';

    params.interestingPointsPC{3} = [7.3956, 2.6947, -3.2416; ...
                                        3.8756, 3.5295, -0.9016; ...
                                        3.9756, 0.7663, 2.5484; ...
                                        4.4656, 2.9874, -0.6228; ...
                                        1.2156, 0.8774, -2.9065; ...
                                        2.2856, 0.0017, 0.4184; ...
                                        3.9056, 0.7663, -0.7916; ...
                                        0.1756, 0.0025, -1.8016; ...
                                        2.9956, 3.3074, -3.2792]';
    params.interestingPointsQuery{3} = [981, 269; ...
                                        758, 35; ...
                                        1098, 663; ...
                                        838, 133; ...
                                        250, 388; ...
                                        442, 664; ...
                                        702, 488; ...
                                        38, 532; ...
                                        528, 112]';

    params.interestingPointsPC{4} = [-3.5344, 2.6774, -3.2205; ...
                                        -11.2077, 2.3874, 1.0384; ...
                                        -7.0966, 0.6574, 1.0484; ...
                                        -5.8644, 3.5374, -2.6916; ...
                                        -2.2544, 0.0000, 1.4684; ...
                                        -3.2690, -0.0026, -0.8816; ...
                                        -7.3044, 3.4974, 2.5959; ...
                                        -11.2159, 2.1774, -3.1516; ...
                                        -1.9107, 3.2574, -2.8416]';
    params.interestingPointsQuery{4} = [1113, 198; ...
                                        406, 250; ...
                                        375, 428; ...
                                        892, 113; ...
                                        193, 706; ...
                                        706, 647; ...
                                        207, 68; ...
                                        742, 303; ...
                                        1279, 19]';

    params.interestingPointsPC{5} = [-8.2444, 1.1374, 8.2766; ...
                                        -10.6837, 3.5356, 5.2184; ...
                                        -7.2519, 1.1474, 4.9484; ...
                                        -7.0244, 0.9659, 6.5084; ...
                                        -8.4444, 3.5361, 8.2084; ...
                                        -7.9444, 3.2362, 8.6884; ...
                                        -9.9644, 1.1574, 4.6253; ...
                                        -11.1544, 2.5489, 8.9884; ...
                                        -11.1744, 3.5193, 8.9655]';
    params.interestingPointsQuery{5} = [152, 515; ...
                                        956, 42; ...
                                        661, 630; ...
                                        175, 646; ...
                                        198, 10; ...
                                        40, 61; ...
                                        1048, 518; ...
                                        456, 284; ...
                                        456, 149]';

    params.interestingPointsPC{6} = [0.8956, 3.5306, 6.4184; ...
                                        10.2356, 2.5374, -3.3136; ...
                                        0.8456, 0.0060, 3.2784; ...
                                        16.8462, 2.0574, 6.6484; ...
                                        3.3268, 3.5376, 4.5884; ...
                                        4.2256, 3.2374, 8.6935; ...
                                        9.8917, 2.2374, 2.2684; ...
                                        2.7356, 0.0050, 5.8384; ...
                                        7.1056, 0.9874, -3.3150]';
    params.interestingPointsQuery{6} = [1194, 64; ...
                                        309, 60; ...
                                        75, 429; ...
                                        894, 475; ...
                                        805, 78; ...
                                        1339, 479; ...
                                        618, 273; ...
                                        649, 691; ...
                                        94, 69]';
end
