function [ params ] = s10eParams(params)

    params.dataset.query.dirname = 'query-s10e';
    params.dataset.query.dir = fullfile(params.dataset.dir, params.dataset.query.dirname); % NOTE: it cannot be extracted to setupParams.m, because we need it in here already
    params.dataset.query.dslevel = 8^-1;

    interestingQueryInd = [1,2,3,4,5,6];
    optimalRotations = {[-89.7792, -1.8236, -1.0584],
                        [-92.9380, -2.3666, -1.4712],
                        [-94.2276, -1.0414, 1.2171],
                        [-89.5798, -0.8083, 0.2376],
                        [-89.3493, -1.0697, -1.4783],
                        [-89.0429, -2.0569, 0.4912]};
    params.optimal.camera.rotation.wrt.marker = containers.Map(interestingQueryInd, optimalRotations);
    %params.optimal.camera.rotation.wrt.marker = containers.Map;

    %params.camera.rotation.wrt.marker = [-90.8195, -1.5277, -0.3437]; % suggested from findOptimalParams
    params.camera.rotation.wrt.marker = [-90.8195, -1.5277, -0.3437]; % my guess
    params.camera.sensor.size = [3024, 4032]; % height, width
    params.camera.fl = 3172.435; % in pixels

    interestingQueryInd = [1,2,3,4,5,6];
    optimalTranslations = {[0.2813; -6.2294; -0.4458],
                            [7.6984; 9.2380; 19.7901],
                            [-13.6836; -1.8501; 21.4086],
                            [-8.2555; 12.4783; 0.9475],
                            [-4.6158; 10.5912; -9.0091],
                            [-1.7383; -0.2856; -6.4469]};
    params.optimal.camera.origin.relative.wrt.marker = containers.Map(interestingQueryInd, optimalTranslations);
    %params.optimal.camera.origin.relative.wrt.marker = containers.Map;
    %params.camera.origin.relative.wrt.marker = [-3.3856; 3.9904; 4.3741]; % suggested from findOptimalParams... (Z makes no sense)
    params.camera.origin.relative.wrt.marker = [-3; 1; -4]; % my guess
    params.camera.originConstant = 0.01;
    params.camera.origin.wrt.marker = params.camera.originConstant * params.camera.origin.relative.wrt.marker;

    params.interestingQueries = ["1.jpg", ...
                                 "13.jpg", ...
                                 "15.jpg", ...
                                 "31.jpg", ...
                                 "33.jpg", ...
                                 "40.jpg"];

    % interestingPointsQuery{i} are 2D projections of points in interestingPointsPC{i}.
    % i is the i-th query in params.interestingQueries
    params.interestingPointsPC{1} = [-7.0750, 0.6574, 1.0184; ...
                                        -11.2115, 3.4974, -3.1916; ...
                                        -2.3376, 3.5374, 2.1006; ...
                                        -2.6544, 0.0011, -0.9216; ...
                                        -1.2162, 2.0474, 2.6284]';
    params.interestingPointsQuery{1} = [2263, 1776; ...
                                        3463, 981; ...
                                        1310, 95; ...
                                        3408, 2582; ...
                                        538, 1023]';

    params.interestingPointsPC{2} = [-7.3044, 0.7803, 7.9484; ...
                                        -4.5044, 0.0021, 7.9784; ...
                                        -5.1044, 0.0073, 7.8784; ...
                                        -8.0844, 1.1415, 8.2843; ...
                                        -6.2544, 0.0139, 8.2538]';
    params.interestingPointsQuery{2} = [3604, 1310; ...
                                        365, 2942; ...
                                        1328, 2781; ...
                                        3908, 830; ...
                                        2379, 2132]';

    params.interestingPointsPC{3} = [-5.2644, -0.0053, -1.3716; ...
                                        -7.9844, 0.6288, -2.3716; ... % the matching is not that accurate here
                                        -6.7344, 0.7455, -2.3295; ...
                                        -6.6118, 1.1374, -2.0431; ...
                                        -7.1374, 0.1474, -2.4916]';
    params.interestingPointsQuery{3} = [2403, 2882; ...
                                        764, 1136; ...
                                        1495, 1185; ...
                                        1405, 905; ...
                                        1394, 1631]';

    params.interestingPointsPC{4} = [-7.0750, 0.6574, 1.0184; ...
                                        -4.5444, 0.0016, -0.2816; ...
                                        -7.3193, 0.0774, 2.6184; ...
                                        -11.3044, 2.3174, 2.8115; ...
                                        -2.1744, 3.5450, 2.1084]';
    params.interestingPointsQuery{4} = [2761, 2544; ...
                                        3902, 2914; ...
                                        2207, 2949; ...
                                        2076, 2122; ...
                                        1452, 676]';

    params.interestingPointsPC{5} = [-11.1744, 3.5074, 8.9756; ...
                                        -2.1944, 3.5288, 5.2184; ...
                                        -6.1244, 3.5374, 4.2684; ... % this projected PC point is kinda weird
                                        -7.5244, 0.9439, 6.5084; ...
                                        -10.6744, 3.5383, 5.8184]'; % this projected PC point is kinda weird
    params.interestingPointsQuery{5} = [2946, 2026; ...
                                        422, 497; ...
                                        3531, 605; ...
                                        3791, 2743; ...
                                        3683, 1400]';

    params.interestingPointsPC{6} = [2.0056, 0.0059, 2.2884; ...
                                        0.8656, 0.0041, 3.2784; ...
                                        -1.8344, 0.0037, 0.6184; ...
                                        1.2656, 0.0037, 3.0084; ...
                                        -7.1650, 0.0197, -0.6816]';
    params.interestingPointsQuery{6} = [3975, 1950; ...
                                        1229, 2934; ...
                                        1212, 215; ...
                                        2000, 2778; ...
                                        235, 49]';
end